%projectile
% This is an example of a while loop. The motion of a projectile under the
% influence of (i) gravity, (ii) drag with a random component is
% determined. The simulation ceases when the projectile hits the ground.
%
% LAST UPDATED by Andy French April 2016

function projectile

%% Inputs %%

%Launch angle /deg
e = 45;

%Launch velocity /ms^-1
u = 10;

%Launch height /m
h = 1;

%Radius of projectile /cm
r = 5;

%Mass of projectile /kg
m = 0.02;

%Density of air /kgm^-3
rho = 1;

%Drag coefficient
cD = 1;

%Random wind speed /ms^-1
w = 20;

%Strength of gravity /ms^-2
g = 9.81;

%Time step for simulation /s
dt = 0.01;

%Define max number of steps (worse case!)
Nmax = 1e6;

%Fontsize for plot
fsize = 18;

%

%Calculate drag constant k
k = 0.5*cD*rho*pi*(r/100)^2;

%Initialize x,y,t,vx,vy vectors, which will grow as the iteration proceeds.
%Since we don't know how many time steps we need, this is unavoidable. It
%will make the program slower though, as memory will have to be allocated
%as we go. This could be a problem if the number of iterations becomes
%large!
x = 0;
y = h;
t = 0;
vx = u*cos(e*pi/180);
vy = u*sin(e*pi/180);

%Loop simulation, stopping if iteration number exceeds Nmax, or if y<0
n=1;
while ( y(n)>=0 ) && ( n<=Nmax )
    
    %Calculate velocity magnitude (squared)
    v2 = vx(n)^2 + vy(n)^2 ;
    
    %Work out random wind direction (radians). rand gives a random number
    %betwen 0 and 1.
    theta = 2*pi*rand;
    
    %Use Newton's Second Law to determine the force in the x and y
    %directions
    Fx = -k*vx(n)*v2/abs(vx(n)) + k*(w^2)*cos(theta);
    Fy = -m*g + -k*vy(n)*v2/abs(vy(n)) + k*(w^2)*sin(theta);
    
    %Hence work out accelerations in x and y
    ax = Fx/m;
    ay = Fy/m;
    
    %Estimate new velocities, assuming consstant acceleration during the
    %time step. Also work out new x and y values, aso assuming constant
    %acceleration motion during the time step dt
    vx(n+1) = vx(n) + ax*dt;
    vy(n+1) = vy(n) + ay*dt;
    x(n+1) = x(n) + vx(n)*dt + 0.5*ax*dt^2;
    y(n+1) = y(n) + vy(n)*dt + 0.5*ay*dt^2;
    
    %Update time and iteration number
    t(n+1) = t(n) +dt;
    n = n + 1;
end

%Plot trajectory
figure('color',[ 1 1 1 ],'name','projectile')
plot(x,y);
xlabel('x','fontsize',fsize);
ylabel('y','fontsize',fsize);
title(['Projectile: Time of flight = ',num2str(t(end)),' seconds.'],'fontsize',fsize)
set(gca,'fontsize',fsize)
xlim([min(x),max(x)]);
ylim([0,max(y)]);
grid on;

%Save a .png file of the plot
filename = ['projectile ',strrep(datestr(now),':','-'),'.png'];
print( gcf, filename, '-dpng','-r300' );

%End of code