%snails
% N snails move at velocity v cm per minute, starting along the vertices of
% a regular N-gon. They always move in order to approach the snail ahead of
% them. This means the N-gon geometry always defines their relative
% separations, but the N-gon rotates at rate v*sin(alpha)/r and its radius
% decreases at rate v*cos(alpha). alpha = 90*(N-2)/N i.e. the angle between
% the side of the N-gon and the radial line.
%
% LAST UPDATED by Andy French April 2016

function snails

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Inputs %%

%Number of snails (and sides of regular N-gon 
%separating the snails)
N = 6;

%Snail speed in cm per minute
v = 5;

%Side length of N-gon /cm
s = 60;

%Define number of time steps for trajectory plot
P = 1000;

%Define angular step size for whirl plot
dtheta = 2*pi/100;

%Maximum number of whirl rotations
max_num_whirls = 5;

%Fontsize for graphs
fsize = 18;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%

%Angle alpha (in radians)
alpha = (pi/2)*(N-2)/N;

%Compute initial radius from cenre of N-gon
r0 = (s/2)/cos(alpha);

%Compute initial angles of the snails (anticlockwise from horizontal);
theta0 = zeros(1,N);
for n=1:N
    theta0(n) = pi/2 + (n-1)*2*pi/N;
end

%Compute time required for snails to reach the centre /mins
T = r0/(v*cos(alpha));

%Define trajectory for snails
t = linspace(0,T,P);
x = zeros(P,N);
y = zeros(P,N);
for n=1:N
    r = r0 - v*t*cos(alpha);
    theta = theta0(n) - tan(alpha) * log( r/r0 );
    x(:,n) = r(:).*cos(theta(:));
    y(:,n) = r(:).*sin(theta(:));
end

%Plot trajectories
figure('color',[1 1 1],'name','Snails of pursuit');
plot(x,y);
axis equal;
xlabel('x /cm','fontsize',fsize)
ylabel('y /cm','fontsize',fsize);
title( ['Snails of pursuit around a ',num2str(N),...
    '-gon. T = ',num2str(T),' mins.' ],'fontsize',fsize );
set(gca,'fontsize',fsize)
hold on;

%Plot starting positions
plot( [x(1,:);x(1,:)],[y(1,:);y(1,:)],'*')
box on
grid on
axis tight

%Plot edges of N-gon;
plot( [x(1,:),x(1,1)],[y(1,:),y(1,1)],'k');

%Save PNG file
print( gcf, ['Snails of pursuit around a ',num2str(N),...
    '-gon T = ',num2str(T),'mins.png' ],'-dpng','-r300' );

%Clear figure and then create a whorl of the N-gon i.e. join up the snail
%trajectories
cla;

%Plot initial N-gon
plot( [x(1,:),x(1,1)],[y(1,:),y(1,1)],'k');
hold on;

%Rotate through angular step size dtheta and work out new snail positions.
%Join these up with straight lines to form a rotated and reduced size N-gon.
delta_theta = 0;
x = zeros(1,N+1);
y = zeros(1,N+1);
while delta_theta < max_num_whirls*2*pi
    delta_theta = delta_theta + dtheta;
    for n=1:N
        theta = theta0(n) + delta_theta;
        r = r0*exp( - delta_theta*cot(alpha) );
        x(n) = r.*cos(theta);
        y(n) = r.*sin(theta);
    end
    x(N+1) = x(1);
    y(N+1) = y(1);
    plot( x,y,'k');
end

%Save PNG file and close figure
print( gcf, ['Snails of pursuit around a ',...
    num2str(N),'-gon whirly.png' ],'-dpng','-r300' );
close(gcf);

%End of code